library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.numeric_std.all;

entity kb_input is
	port(
		clk:			in std_logic;	-- main clock
		rst:			in std_logic;	-- reset
		psClk:		in std_logic;	-- keyboard clock
		psData:		in std_logic;	-- keyboard data
		scancode:	out std_logic_vector(7 downto 0);	-- key scancode
		rdy:			out std_logic	-- scancode ready pulse
	);
end entity;
	
architecture arch of kb_input is
signal		sc_r:				std_logic_vector(9 downto 0);	-- scancode shift register
constant	clkFreq:		natural := 50_000;	-- main clock frequency (KHz)
constant	psClkFreq:	natural := 10;			-- keyboard clock frequency (KHz)
constant	timeout:		natural := clkFreq / psClkFreq;	-- psClk quiet timeout
subtype		counter is natural range 0 to timeout;
signal		cnt_r: 			counter;
signal		scRdy_r:		std_logic;	-- scan code is ready flag
signal		rdy_r:			std_logic;	-- output scan code is ready flag
signal		keyrel_r:		std_logic;
constant	keyRelease:	std_logic_vector(7 downto 0) := "11110000";
begin

	-- this process places the keyboard scancode into the shift register
	process(psClk,rst)
	begin
		-- async. reset of scancode ready flag
		if rst = '1' then
			sc_r <= (others=>'0');
		-- accept keyboard data on falling edge of keyboard clock
		elsif psClk'event and psClk='0' then
			-- key data arrives LSB first so right-shift it into MSB of register
			sc_r <= psData & sc_r(9 downto 1);
		end if;
	end process;
	
	-- key scancode is in the lower 8-bits of the shift register
	scancode <= sc_r(scancode'range);	-- output scancode

	-- this process detects the end of the scancode by looking
	-- for the absence of keyboard clock pulses
	process(clk,rst)
	begin
		if rst = '1' then
			cnt_r <= 0;
			scRdy_r <= '0';
		elsif clk'event and clk = '1' then
			scRdy_r <= '0';	-- by default, no key scancode is ready for output
			if psClk = '0' then
				-- reset the timeout register whenever the keyboard clock pulses low
				cnt_r <= 0;
			elsif cnt_r /= timeout then
				-- increment the timeout counter if the keyvoard clock is high
				-- and the counter hasn't reached the timeout value yet
				cnt_r <= cnt_r + 1;
				if cnt_r = timeout-1 then
					-- signal that a key scancode is ready when the counter is
					-- equal to one less than the timeout value
					scRdy_r <= '1';	-- the rdy signal will pulse for a single clock period
				end if;
			end if;
		end if;
	end process;

	-- this process detects when the keyboard key is released and
	-- signals when the scancode for the released key is ready
	process(clk)
	begin
		if clk'event and clk = '1' then
			rdy_r <= '0';	-- by default, no key scancode is ready for output
			if scRdy_r = '1' then
				-- check the scancode register when a code is ready
				if sc_r(7 downto 0) = keyRelease then
					-- set flag if the keyRelease prefix is detected
					keyrel_r <= '1';
				elsif keyrel_r = '1' then
					-- end up here on next scancode received after key release prefix
					rdy_r <= '1';			-- released key scancode is in the scancode register
					keyrel_r <= '0';	-- reset the key release flag
				end if;
			end if;
		end if;
	end process;
	
	rdy <= rdy_r;	-- signal that a key scancode is ready

end architecture;
